@@ -1,6 +1,3 @@
-"""
-Plugin for Pyramid apps to submit errors to Rollbar
-"""
 from __future__ import absolute_import
 from __future__ import unicode_literals
 
@@ -18,6 +15,7 @@ import traceback
 import types
 import uuid
 import wsgiref.util
+import warnings
 
 import requests
 import six
@@ -25,7 +23,7 @@ import six
 from rollbar.lib import events, filters, dict_merge, parse_qs, text, transport, urljoin, iteritems, defaultJSONEncode
 
 
-__version__ = '0.15.1'
+__version__ = '0.16.2'
 __log_name__ = 'rollbar'
 log = logging.getLogger(__log_name__)
 
@@ -91,6 +89,22 @@ try:
 except ImportError:
     AppEngineFetch = None
 
+try:
+    from starlette.requests import Request as StarletteRequest
+except ImportError:
+    StarletteRequest = None
+
+try:
+    from fastapi.requests import Request as FastAPIRequest
+except ImportError:
+    FastAPIRequest = None
+
+try:
+    import httpx
+except ImportError:
+    httpx = None
+
+AsyncHTTPClient = httpx
 
 def passthrough_decorator(func):
     def wrap(*args, **kwargs):
@@ -105,6 +119,23 @@ except ImportError:
 try:
     import treq
     from twisted.python import log as twisted_log
+    from twisted.web.iweb import IPolicyForHTTPS
+    from twisted.web.client import BrowserLikePolicyForHTTPS, Agent
+    from twisted.internet.ssl import CertificateOptions
+    from twisted.internet import task, defer, ssl, reactor
+    from zope.interface import implementer
+    
+    @implementer(IPolicyForHTTPS)
+    class VerifyHTTPS(object):
+        def __init__(self):
+            # by default, handle requests like a browser would
+            self.default_policy = BrowserLikePolicyForHTTPS()
+
+        def creatorForNetloc(self, hostname, port):
+            # check if the hostname is in the the whitelist, otherwise return the default policy
+            if not SETTINGS['verify_https']:
+                return ssl.CertificateOptions(verify=False)
+            return self.default_policy.creatorForNetloc(hostname, port)
 
     def log_handler(event):
         """
@@ -147,7 +178,9 @@ def get_request():
     # TODO(cory): add in a generic _get_locals_request() which
     # will iterate up through the call stack and look for a variable
     # that appears to be valid request object.
-    for fn in (_get_bottle_request,
+    for fn in (_get_fastapi_request,
+               _get_starlette_request,
+               _get_bottle_request,
                _get_flask_request,
                _get_pyramid_request,
                _get_pylons_request):
@@ -189,6 +222,26 @@ def _get_pylons_request():
     return request
 
 
+def _get_starlette_request():
+    # Do not modify the returned object
+
+    if StarletteRequest is None:
+        return None
+
+    from rollbar.contrib.starlette import get_current_request
+    return get_current_request()
+
+
+def _get_fastapi_request():
+    # Do not modify the returned object
+
+    if FastAPIRequest is None:
+        return None
+
+    from rollbar.contrib.fastapi import get_current_request
+    return get_current_request()
+
+
 BASE_DATA_HOOK = None
 
 agent_log = None
@@ -222,7 +275,7 @@ SETTINGS = {
     'root': None,  # root path to your code
     'branch': None,  # git branch name
     'code_version': None,
-    'handler': 'thread',  # 'blocking', 'thread', 'agent', 'tornado', 'gae' or 'twisted'
+    'handler': 'default',  # 'blocking', 'thread' (default), 'async', 'agent', 'tornado', 'gae', 'twisted' or 'httpx'
     'endpoint': DEFAULT_ENDPOINT,
     'timeout': DEFAULT_TIMEOUT,
     'agent.log_file': 'log.rollbar',
@@ -252,6 +305,7 @@ SETTINGS = {
         'safe_repr': True,
         'scrub_varargs': True,
         'sizes': DEFAULT_LOCALS_SIZES,
+        'safelisted_types': [],
         'whitelisted_types': []
     },
     'verify_https': True,
@@ -330,6 +384,10 @@ def init(access_token, environment='production', scrub_fields=None, url_fields=N
     if SETTINGS.get('handler') == 'agent':
         agent_log = _create_agent_log()
 
+    if not SETTINGS['locals']['safelisted_types'] and SETTINGS['locals']['whitelisted_types']:
+        warnings.warn('whitelisted_types deprecated use safelisted_types instead', DeprecationWarning)
+        SETTINGS['locals']['safelisted_types'] = SETTINGS['locals']['whitelisted_types']
+
     # We will perform these transforms in order:
     # 1. Serialize the payload to be all python built-in objects
     # 2. Scrub the payloads based on the key suffixes in SETTINGS['scrub_fields']
@@ -337,7 +395,7 @@ def init(access_token, environment='production', scrub_fields=None, url_fields=N
     # 4. Optional - If local variable gathering is enabled, transform the
     #       trace frame values using the ShortReprTransform.
     _serialize_transform = SerializableTransform(safe_repr=SETTINGS['locals']['safe_repr'],
-                                                 whitelist_types=SETTINGS['locals']['whitelisted_types'])
+                                                 safelist_types=SETTINGS['locals']['safelisted_types'])
     _transforms = [
         ScrubRedactTransform(),
         _serialize_transform,
@@ -459,18 +517,27 @@ def send_payload(payload, access_token):
     Available handlers:
     - 'blocking': calls _send_payload() (which makes an HTTP request) immediately, blocks on it
     - 'thread': starts a single-use thread that will call _send_payload(). returns immediately.
+    - 'async': calls _send_payload_async() (which makes an async HTTP request using default async handler)
     - 'agent': writes to a log file to be processed by rollbar-agent
     - 'tornado': calls _send_payload_tornado() (which makes an async HTTP request using tornado's AsyncHTTPClient)
     - 'gae': calls _send_payload_appengine() (which makes a blocking call to Google App Engine)
-    - 'twisted': calls _send_payload_twisted() (which makes an async HTTP reqeust using Twisted and Treq)
+    - 'twisted': calls _send_payload_twisted() (which makes an async HTTP request using Twisted and Treq)
+    - 'httpx': calls _send_payload_httpx() (which makes an async HTTP request using HTTPX)
     """
     payload = events.on_payload(payload)
     if payload is False:
         return
 
-    payload_str = _serialize_payload(payload)
+    if sys.version_info >= (3, 6):
+        from rollbar.lib._async import get_current_handler
+        handler = get_current_handler()
+    else:
+        handler = SETTINGS.get('handler')
 
-    handler = SETTINGS.get('handler')
+    if handler == 'twisted':
+        payload['data']['framework'] = 'twisted'
+
+    payload_str = _serialize_payload(payload)
     if handler == 'blocking':
         _send_payload(payload_str, access_token)
     elif handler == 'agent':
@@ -490,11 +557,21 @@ def send_payload(payload, access_token):
             log.error('Unable to find Treq')
             return
         _send_payload_twisted(payload_str, access_token)
+    elif handler == 'httpx':
+        if httpx is None:
+            log.error('Unable to find HTTPX')
+            return
+        _send_payload_httpx(payload_str, access_token)
+    elif handler == 'async':
+        if AsyncHTTPClient is None:
+            log.error('Unable to find async handler')
+            return
+        _send_payload_async(payload_str, access_token)
+    elif handler == 'thread':
+        _send_payload_thread(payload_str, access_token)
     else:
         # default to 'thread'
-        thread = threading.Thread(target=_send_payload, args=(payload_str, access_token))
-        _threads.put(thread)
-        thread.start()
+        _send_payload_thread(payload_str, access_token)
 
 
 def search_items(title, return_fields=None, access_token=None, endpoint=None, **search_fields):
@@ -843,7 +920,7 @@ def _add_person_data(data, request):
 
 def _build_person_data(request):
     """
-    Returns a dictionary describing the logged-in user using data from `request.
+    Returns a dictionary describing the logged-in user using data from `request`.
 
     Try request.rollbar_person first, then 'user', then 'user_id'
     """
@@ -855,7 +932,12 @@ def _build_person_data(request):
         else:
             return None
 
-    if hasattr(request, 'user'):
+    if StarletteRequest:
+        from rollbar.contrib.starlette.requests import hasuser
+    else:
+        def hasuser(request): return True
+
+    if hasuser(request) and hasattr(request, 'user'):
         user_prop = request.user
         user = user_prop() if callable(user_prop) else user_prop
         if not user:
@@ -1060,7 +1142,6 @@ def _get_actual_request(request):
 def _build_request_data(request):
     """
     Returns a dictionary containing data from the request.
-    Can handle webob or werkzeug-based request objects.
     """
 
     # webob (pyramid)
@@ -1099,6 +1180,14 @@ def _build_request_data(request):
     if isinstance(request, dict) and 'wsgi.version' in request:
         return _build_wsgi_request_data(request)
 
+    # FastAPI (built on top of Starlette, so keep the order)
+    if FastAPIRequest and isinstance(request, FastAPIRequest):
+        return _build_fastapi_request_data(request)
+
+    # Starlette (should be the last one for Starlette based frameworks)
+    if StarletteRequest and isinstance(request, StarletteRequest):
+        return _build_starlette_request_data(request)
+
     return None
 
 
@@ -1278,6 +1367,53 @@ def _build_wsgi_request_data(request):
 
     return request_data
 
+def _build_starlette_request_data(request):
+    from starlette.datastructures import UploadFile
+
+    request_data = {
+        'url': str(request.url),
+        'GET': dict(request.query_params),
+        'headers': dict(request.headers),
+        'method': request.method,
+        'user_ip': _starlette_extract_user_ip(request),
+        'params': dict(request.path_params),
+    }
+
+    if hasattr(request, '_form'):
+        request_data['POST'] = {
+            k: v.filename if isinstance(v, UploadFile) else v
+            for k, v in request._form.items()
+        }
+        request_data['files_keys'] = [
+            field.filename
+            for field in request._form.values()
+            if isinstance(field, UploadFile)
+        ]
+
+    if hasattr(request, '_body'):
+        body = request._body.decode()
+    else:
+        body = None
+
+    if body and SETTINGS['include_request_body']:
+        request_data['body'] = body
+
+    if hasattr(request, '_json'):
+        request_data['json'] = request._json
+    elif body:
+        try:
+            request_data['json'] = json.loads(body)
+        except json.JSONDecodeError:
+            pass
+
+    # Filter out empty values
+    request_data = {k: v for k, v in request_data.items() if v}
+
+    return request_data
+
+def _build_fastapi_request_data(request):
+    return _build_starlette_request_data(request)
+
 
 def _filter_ip(request_data, capture_ip):
     if 'user_ip' not in request_data or capture_ip == True:
@@ -1368,6 +1504,12 @@ def _send_payload(payload_str, access_token):
         pass
 
 
+def _send_payload_thread(payload_str, access_token):
+    thread = threading.Thread(target=_send_payload, args=(payload_str, access_token))
+    _threads.put(thread)
+    thread.start()
+
+
 def _send_payload_appengine(payload_str, access_token):
     try:
         _post_api_appengine('item/', payload_str, access_token=access_token)
@@ -1467,7 +1609,6 @@ def _send_payload_twisted(payload_str, access_token):
     except Exception as e:
         log.exception('Exception while posting item %r', e)
 
-
 def _post_api_twisted(path, payload_str, access_token=None):
     def post_data_cb(data, resp):
         resp._content = data
@@ -1488,10 +1629,28 @@ def _post_api_twisted(path, payload_str, access_token=None):
         encoded_payload = payload_str.encode('utf8')
     except (UnicodeDecodeError, UnicodeEncodeError):
         encoded_payload = payload_str
-    d = treq.post(url, encoded_payload, headers=headers,
+
+    treq_client = treq.client.HTTPClient(Agent(reactor, contextFactory=VerifyHTTPS()))
+    d = treq_client.post(url, encoded_payload, headers=headers,
                   timeout=SETTINGS.get('timeout', DEFAULT_TIMEOUT))
     d.addCallback(post_cb)
 
+def _send_payload_httpx(payload_str, access_token):
+    from rollbar.lib._async import call_later, _post_api_httpx
+    try:
+        call_later(_post_api_httpx('item/', payload_str,
+                                   access_token=access_token))
+    except Exception as e:
+        log.exception('Exception while posting item %r', e)
+
+
+
+def _send_payload_async(payload_str, access_token):
+    try:
+        _send_payload_httpx(payload_str, access_token=access_token)
+    except Exception as e:
+        log.exception('Exception while posting item %r', e)
+
 
 def _send_failsafe(message, uuid, host):
     body_message = ('Failsafe from pyrollbar: {0}. Original payload may be found '
@@ -1581,15 +1740,18 @@ def _parse_response(path, access_token, params, resp, endpoint=None):
             return Result(access_token, path, params, result)
 
 
-def _extract_user_ip(request):
-    # some common things passed by load balancers... will need more of these.
-    real_ip = request.headers.get('X-Real-Ip')
-    if real_ip:
-        return real_ip
+def _extract_user_ip_from_headers(request):
     forwarded_for = request.headers.get('X-Forwarded-For')
     if forwarded_for:
         return forwarded_for
-    return request.remote_addr
+    real_ip = request.headers.get('X-Real-Ip')
+    if real_ip:
+        return real_ip
+    return None
+
+
+def _extract_user_ip(request):
+    return _extract_user_ip_from_headers(request) or request.remote_addr
 
 
 def _wsgi_extract_user_ip(environ):
@@ -1600,3 +1762,7 @@ def _wsgi_extract_user_ip(environ):
     if real_ip:
         return real_ip
     return environ['REMOTE_ADDR']
+
+
+def _starlette_extract_user_ip(request):
+    return request.client.host or _extract_user_ip_from_headers(request)
